from yta_general_utils.file_downloader import download_image
from yta_general_utils.experimental.chrome_scrapper import start_chrome, go_to_and_wait_loaded
from random import randint
from datetime import datetime

import requests
import time
import os

PROJECT_ABSOLUTE_PATH = os.getenv('PROJECT_ABSOLUTE_PATH')

class AIFumes():
    """
    They are implementing a lot of models in their webpage: https://fumesai.web.app/. They are also
    working on an API that I want to use :].
    """
    HUGGIN_FACE_IMAGE_MODELS_DEMO_HEADERS = {
        'authority': 'flask-hello-world-murex-sigma.vercel.app',
        'accept': '*/*',
        'accept-language': 'es-ES,es;q=0.9',
        'content-type': 'application/json',
        'origin': 'https://fumesai-text2image-models-demo.static.hf.space',
        'referer': 'https://fumesai-text2image-models-demo.static.hf.space/',
        'sec-ch-ua': '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'cross-site',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36',
    }

    def generate(self, prompt, negative_prompt, output_filename):
        """
        This is the default generating method that will use the method I manually chose from
        code (maybe because someone is down due to external issues).
        """
        # I force this negative prompt by now
        negative_prompt = ',(bad hands, bad anatomy, bad body, bad face, bad teeth, bad arms, bad legs, deformities:1.3),poorly drawn,deformed hands,deformed fingers,deformed faces,deformed eyes,mutated fingers,deformedbody parts,mutated body parts,mutated hands, disfigured,oversaturated,bad anatom,cropped, worst quality, low quality, jpeg artifacts, ugly, duplicate, morbid, mutilated, out of frame, extra fingers, mutated hands, poorly drawn hands, poorly drawn face, mutation, deformed, blurry, dehydrated, bad anatomy, bad proportions, extra limbs, cloned face, disfigured, gross proportions, deformed limbs, missing arms, missing legs, extra arms, extra legs, fused fingers, too many fingers, long neck,deformed eyes'

        self.generate_leonardo(prompt, negative_prompt, output_filename)

    def __get_recaptcha_token(self):
        """
        TODO: This is not working. This pretends to get a reCAPTCHA token to be able to do other 
        requests in the webpage in which it is implemented.
        """
        
        # Open (https://huggingface.co/spaces/FumesAI/Leonardo-AI-image-Creator-UPDATED) with crome
        # Alternative page: https://fumesai-leonardo-ai-image-creator-updated.static.hf.space/index.html
        URL = 'https://fumesai-leonardo-ai-image-creator-updated.static.hf.space/index.html'
        chromedriver = start_chrome(False, False, False)
        go_to_and_wait_loaded(chromedriver, URL)
        time.sleep(10)

        # #time.sleep(10)
        # # Obtaining cookies
        # cookie_3PAPISID = chromedriver.get_cookie('__Secure-3PAPISID')
        # cookie_3PSID = chromedriver.get_cookie('__Secure-3PSID')
        # cookie_1P_JAR = chromedriver.get_cookie('1P_JAR')
        # cookie_NID = chromedriver.get_cookie('NID')
        # cookie_3PSIDTS = chromedriver.get_cookie('__Secure-3PSIDTS')
        # cookie_3PSIDCC = chromedriver.get_cookie('__Secure-3PSIDCC')
        # print(chromedriver.get_cookies())
        # #time.sleep(30)

        from selenium.webdriver.common.by import By

        
        # # Get iframe title='reCAPTCHA' information for the request
        recaptcha_iframe = chromedriver.find_element(By.XPATH, '//iframe[contains(@title, "reCAPTCHA")]')
        recaptcha_url = recaptcha_iframe.get_attribute('src')
        recaptcha_hidden_input = recaptcha_iframe.find_element(By.ID, 'recaptcha-token')
        recaptcha_token_value = recaptcha_hidden_input.get_attribute('value')
        # recaptcha_url = chromedriver.find_element(By.XPATH, '//iframe[contains(@title, "reCAPTCHA")]').get_attribute('src')
        # print(recaptcha_url)
        # chromedriver.switch_to.new_window()
        # go_to_and_wait_loaded(chromedriver, recaptcha_url)
        # #time.sleep(3)
        # recaptcha_token_value = chromedriver.find_element(By.ID, 'recaptcha-token').get_attribute('value')
        # #recaptcha_token_value = chromedriver.execute_script('document.getElementById("recaptcha-token").value')
        # chromedriver.close()
        # #recaptcha_token = chromedriver.find_element(By.ID, 'recaptcha-token').get_attribute('value')
        
        # #recaptcha_token_value = chromedriver.execute_script('document.getElementById("recaptcha-token").value')
        # print(recaptcha_token_value)
        elements = recaptcha_url.split('&')
        k = elements[1].split('=')[1]
        co = elements[2].split('=')[1]
        hl = elements[3].split('=')[1]
        v = elements[4].split('=')[1]
        size = elements[5].split('=')[1]
        cb = elements[6].split('=')[1]
        # # It is like 'https://www.google.com/recaptcha/api2/anchor?ar=1&k=6Leqa5cpAAAAABVhh6FGouusHKaPjYz65-0Yy8kS&co=aHR0cHM6Ly9mdW1lc2FpLWxlb25hcmRvLWFpLWltYWdlLWNyZWF0b3ItdXBkYXRlZC5zdGF0aWMuaGYuc3BhY2U6NDQz&hl=es&v=QquE1_MNjnFHgZF4HPsEcf_2&size=invisible&cb=6beu3j2e4aw1'

        # cookies = {
        #     '__Secure-3PAPISID': cookie_3PAPISID,
        #     '__Secure-3PSID': cookie_3PSID,
        #     '1P_JAR': cookie_1P_JAR,
        #     'NID': cookie_NID,
        #     # These two bellow change
        #     '__Secure-3PSIDTS': cookie_3PSIDTS,
        #     '__Secure-3PSIDCC': cookie_3PSIDCC,
        # }

        # #print(cookies)
        # #chromedriver.close()

        # #exit()
        
        # cookies = {
        #     '__Secure-3PAPISID': '_WzuX9CrPc_FkcnN/AuyTLnEH9KU6HehCw',
        #     '__Secure-3PSID': 'g.a000gwihvpnyIXm_lkiIHWDAxR5zEc3nmpehpn5rkkF0Qi4iYBDKqylsZenvw3HYCckDh_9QcQACgYKAeYSAQASFQHGX2Mitln4s1aKi3-7FYEiC43LCRoVAUF8yKoWA4M9yXlK-ESydg8qZjEO0076',
        #     '1P_JAR': '2024-03-14-10',
        #     'NID': '512=LovqwJ2ZvqbyuntKUMl91ck_Hox3CiGtqz51sDb1BJzdsi9gzIROjfPd_jjylMiGBQyHX5NK3WWeFn2OwqwhSfu4Rto6N0A3f2kMx58bB9FEI1AyWSiEYvULI01XbicEt40gzY0W8I_6HmbMW6CgCh5qVqqD1lxsJbYhVIiceSFokaydCE2_ztwJNcF5i2_cOAUVwLtoPDPkQz076rYc4V3uN3gcgb0yaPg2IfbnKrNQv8PFgg015gi05QCXFM52uaq4M7hxWxDd932CT1d42Tf4FxI18Tgk2Pptz1pAUfdZO05GwrV2jltWAcah2eebT5E7g35arH1l',
        #     # These two bellow change but I can get from chromedriver
        #     '__Secure-3PSIDTS': 'sidts-CjIBYfD7Z5_Rcj0Q8U1RhodY_Sa-XvhygWai2kUJmcrP92A4ga6ffTrznuhYL4Dy0Ae4YhAA',
        #     '__Secure-3PSIDCC': 'AKEyXzWYLPS1hnmEdQBdi3nQf6B1XISaMGqbMQlde94fkLs-ttnr_fylRh76sYAQkwKX-GkzFOY',
        # }
        
        # headers = {
        #     'authority': 'www.google.com',
        #     'accept': '*/*',
        #     'accept-language': 'es-ES,es;q=0.9',
        #     'content-type': 'application/x-protobuffer',
        #     # 'cookie': '__Secure-3PAPISID=_WzuX9CrPc_FkcnN/AuyTLnEH9KU6HehCw; __Secure-3PSID=g.a000gwihvpnyIXm_lkiIHWDAxR5zEc3nmpehpn5rkkF0Qi4iYBDKqylsZenvw3HYCckDh_9QcQACgYKAeYSAQASFQHGX2Mitln4s1aKi3-7FYEiC43LCRoVAUF8yKoWA4M9yXlK-ESydg8qZjEO0076; 1P_JAR=2024-03-14-10; NID=512=LovqwJ2ZvqbyuntKUMl91ck_Hox3CiGtqz51sDb1BJzdsi9gzIROjfPd_jjylMiGBQyHX5NK3WWeFn2OwqwhSfu4Rto6N0A3f2kMx58bB9FEI1AyWSiEYvULI01XbicEt40gzY0W8I_6HmbMW6CgCh5qVqqD1lxsJbYhVIiceSFokaydCE2_ztwJNcF5i2_cOAUVwLtoPDPkQz076rYc4V3uN3gcgb0yaPg2IfbnKrNQv8PFgg015gi05QCXFM52uaq4M7hxWxDd932CT1d42Tf4FxI18Tgk2Pptz1pAUfdZO05GwrV2jltWAcah2eebT5E7g35arH1l; __Secure-3PSIDTS=sidts-CjIBYfD7Z5_Rcj0Q8U1RhodY_Sa-XvhygWai2kUJmcrP92A4ga6ffTrznuhYL4Dy0Ae4YhAA; __Secure-3PSIDCC=AKEyXzWYLPS1hnmEdQBdi3nQf6B1XISaMGqbMQlde94fkLs-ttnr_fylRh76sYAQkwKX-GkzFOY',
        #     'origin': 'https://www.google.com',
        #     'referer': 'https://www.google.com/recaptcha/api2/anchor?ar=1&k=' + k + '&co=' + co + '&hl=es&v=' + v + '&size=invisible&cb=' + cb,
        #     'sec-ch-ua': '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
        #     'sec-ch-ua-mobile': '?0',
        #     'sec-ch-ua-platform': '"Windows"',
        #     'sec-fetch-dest': 'empty',
        #     'sec-fetch-mode': 'cors',
        #     'sec-fetch-site': 'same-origin',
        #     'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36',
        #     'x-client-data': 'CJS2yQEIorbJAQipncoBCJzzygEIlqHLAQic/swBCIWgzQEY9cnNARjSgs4B',
        # }

        # params = {
        #     'k': k,
        # }

        # "03AFcWeA5iq5KRnX6G4cEa9vdSi7xmlnqnaN7eO0mfR-B3Py79GbkMepAVy01OF9PM0TYms1Ze00HVIiaDwBaTqYEQvHxy1Old1BhD2JBgK0eQh-S_AnscrLRCOv_003ceP-w3PlWdNF0MQA9FRjR0_qJrecpPbhfrDRsaxTFD3aJYeIAGsw7QabomNYWOdRTGfI1ALYB94YOdsV_fLbXG1sAgHgbuZrzGGSpm_qRb4U9DHQSSSdzX1bpMvMpNMrSAu8AacqZcYBuZtuqO8IFS2opknsZgrtiaHaS_5MNk94gI9bYQ1QMlXjksjaBwzZvGfl9MXLIGEtKk5MTMoje3pu9Ckg0ChL3YD1LHIMTTVWEBi1wNlQCOHfbkb3Zy2sIheZxA87UJaevv9fcQqIPM6A30sySsJ-dPLsmqC0t4K0V9RWBLzqgUadmO6Kwu2BgYML021ReKHq3BWJhhihhWxp0dqvvb2_95wRcKZ_mS4cyw5ygsdBwbjHUPmY68TLQ4ZPAH7b9WqbnhsrQLtlA_vcekuKVk6HUEH0gP_rbZ4GCu0ZaTNuHwf4vnXzgnzzbK_QkEuvqWem26tcRybwvIbYi9LYv6piES-Mk1rmk2eSbT9Pd53AgK_Blo99eiQW4aRfYZi7joU6ZoJPG7nvb9n5y0X_lvavNHv5JO1ZAQTW8eM6qFmcLBB1dOQ2GA0CfmjxkZaszoOiXap4TcCrcAYS_SHn5kbMnctyezzjdpkb5VbPOjO9Rrfu5n7tGSlyh8Qjsw1xTXH_r0wXCmZdWMRC10Lf7KvLLE58PCV9axAWKE8Xn3jiab694nw4srIPMCPIUXlOVXWk-YMdQkH8I3ZetzYUSmq4qPlVWN-lb4-r3nLWcKi6VGR2aI6WgTBUHTWY6HoNEOF8AJfGsXrvw-qMEBUWw9066bcD9ErG6Lk-cHzU7HsmhNvRvCS86WOgNHtwutWpurWRSXZ7W4W2Nkzyg7ypfExRXYE8HL0JVBP4OuWeIaeCVSzhENmoQnuMC8v3aru6j5Wur-73TTRorwfFQoEng-BE4T3vKzwQnirmPso8I7a7yZlNgERvgsO4kQoRSfZwy8ucq49HPhOKBMMasfmMP6FmIIlH-HiKiOLUFLO2iR7oPgkbCM5wOMROEDSGT1bRIpDN-bfE39wI_ypIUFJT0vhGlz7a89t_WmP06oEZW2pxAI_l9cDrTLQZvHYLfJ22ZG4s7EGKKGb3EB1mV2qOPM9xb0MwRthQaNDI2I-555cx2jfwe-0Qw2bl1myvLqL8-IIV0qh7rIApJSu1sf1oe7AvM3Tp6FbRGliWb08Ai2wkVc1AyY0N_yO0hHjtN6CsLMfayXzeBvon0xxi1_8ewTvZtg-svxTnmQDsJxolhp20UnA8xxPYg7vSToYO5dKIzOR0ntW_N_VGO_AkeZ_d1OIa-BKoguWPSgipusb2pcdMo6JOsUZqETmvJt_l-zzJT0sY2AhYHUf_nWZZ4CtQA2yG6CcxtLdOsH20KKyXmraiDlJkPbjyfUEtfUuwERwvTK5A4n5_Dx1FKTg6Feh_NXBut8m-YUzE5FAjJ4rCMK3VFw7ZIcxWsFHZtsMj5VYcZ4NxBVcWnNDeCthMeJ1VNDBwrkwhS6oIiwR_dZ4BKeKrxn4DDk_WQJE61YCK95sej5XtvTj0gOYkTiUEjzeQqi7n9w_DqAVwIiNguoIBCCc-Hs4gTMVj7J0ZgfkaZRA3S38AjqTlo5P-RVHbBpC5mElmGwtg"


        # # Where does this come from?
        # data = recaptcha_token_value

        # #response = requests.post('https://www.google.com/recaptcha/api2/reload', params = params, cookies = cookies, headers = headers, data = data)

        # #print(response.content)
        # #token = response.json()['1']

        token = chromedriver.execute_script('grecaptcha.execute("' + k + '");')
        print(token)

        #print(cookies)
        chromedriver.close()

        return token

    def generate_leonardo(self, prompt, negative_prompt = '', output_filename = 'tmp_fumes_leonardo.png'):
        #URL = 'https://fumesai-leonardo-ai-image-creator-updated.static.hf.space/index.html'
        URL = 'https://fumesai.web.app/img'
        chromedriver = start_chrome(True, False, False)
        go_to_and_wait_loaded(chromedriver, URL)

        from selenium.webdriver.common.by import By
        from selenium.webdriver.support.ui import WebDriverWait
        from selenium.webdriver.support import expected_conditions as EC
        from utils.autogui_utils import move_cursor_to
        from random import randint

        prompt_input = chromedriver.find_element(By.ID, 'inp')
        # Write input
        prompt_input.send_keys(prompt)
        time.sleep(1)
        move_cursor_to(randint(400, 1900), randint(100, 900))
        settings_button = chromedriver.find_element(By.XPATH, "//button[contains(text(), 'Setting')]")
        settings_button.click()
        time.sleep(1)
        move_cursor_to(randint(100, 1800), randint(50, 800))
        negative_prompt_input = chromedriver.find_element(By.ID, 'np')

        negative_prompt = ',(bad hands, bad anatomy, bad body, bad face, bad teeth, bad arms, bad legs, deformities:1.3),poorly drawn,deformed hands,deformed fingers,deformed faces,deformed eyes,mutated fingers,deformedbody parts,mutated body parts,mutated hands, disfigured,oversaturated,bad anatom,cropped, worst quality, low quality, jpeg artifacts, ugly, duplicate, morbid, mutilated, out of frame, extra fingers, mutated hands, poorly drawn hands, poorly drawn face, mutation, deformed, blurry, dehydrated, bad anatomy, bad proportions, extra limbs, cloned face, disfigured, gross proportions, deformed limbs, missing arms, missing legs, extra arms, extra legs, fused fingers, too many fingers, long neck,deformed eyes'
        
        if negative_prompt:
            negative_prompt_input.send_keys(negative_prompt)
        time.sleep(1)
        move_cursor_to(randint(100, 1700), randint(100, 800))
        photography_style_radio_button = chromedriver.find_element(By.XPATH, "//input[contains(@value, 'photo')]").click()
        time.sleep(1)
        move_cursor_to(randint(200, 1600), randint(200, 780))

        # Scroll down
        chromedriver.execute_script('window.scrollTo(' + str(0) + ', ' + str(999) + ')')

        landscape_radio_button = chromedriver.find_element(By.XPATH, "//input[contains(@value, '16:9')]").click()
        time.sleep(1)
        move_cursor_to(120, 550)
        close_button = chromedriver.find_element(By.ID, 'close').click()
        time.sleep(1)
        chromedriver.execute_script('window.scrollTo(' + str(0) + ', ' + str(0) + ')')
        move_cursor_to(randint(700, 1700), randint(250, 800))
        for i in range(3):
            move_cursor_to(randint(0, 1000), randint(0, 1000))
        create_button = chromedriver.find_element(By.ID, 'create').click()
        time.sleep(1)

        image_urls = []
        for i in range(100):
            print('Looking for images')
            try:
                images = chromedriver.find_elements(By.TAG_NAME, 'img')
                # Writing to sample.json
                if len(images) > 0:
                    for image in images:
                        image_url = image.get_attribute('src')
                        image_urls.append(image_url)
                    break
            except:
                pass
            time.sleep(1)
            if i % 10 == 0:
                chromedriver.execute_script('window.scrollTo(' + str(randint(0, 70)) + ', ' + str(999) + ')')

        if len(image_urls) > 0:
            print('Downloading ' + image_urls[0])
            # We secure the one we want
            download_image(image_urls[0], output_filename)
            # But then we store all generated images
            for image_url in image_urls:
                print('Downloading ' + image_url)
                delta = (datetime.now() - datetime(1970, 1, 1))
                image_filename = 'leonardo_' + str(int(delta.total_seconds())) + str(randint(0, 10000)) + '_' + str(randint(0, 1000)) + '.png'
                print('Downloading to local leonardo images folder.')
                try:
                    download_image(image_url, PROJECT_ABSOLUTE_PATH + 'leonardo_images/' + image_filename)
                    with open(PROJECT_ABSOLUTE_PATH + 'leonardo_images/leonardo_generated_images.txt', 'a') as outfile:
                        outfile.write(image_url + '\n')
                        outfile.close()
                except:
                    pass

        chromedriver.close()

    def __generate_leonardo_inactive(self, prompt, negative_prompt = '', output_filename = 'tmp_fumes_leonardo.png'):
        """
        This is a generation created in this HugginFace space: https://huggingface.co/spaces/FumesAI/Leonardo-AI-image-Creator-UPDATED

        [ ! ] TODO: This is not working due to reCAPTCHA token needed in AIFumes API request.
        I was not able to get it an doing a good request, so I decided to implement an scrapper
        to be able to get images.
        """
        HEADERS = {
            'authority': 'fumes-api.onrender.com',
            'accept': '*/*',
            'accept-language': 'es-ES,es;q=0.9',
            'content-type': 'application/json',
            'origin': 'https://fumesai-leonardo-ai-image-creator-updated.static.hf.space',
            'referer': 'https://fumesai-leonardo-ai-image-creator-updated.static.hf.space/',
            'sec-ch-ua': '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'cross-site',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36',
        }

        # TODO: Investigate all available 'styles' to integrate as parameter
        STYLES = [
            'PHOTOGRAPHY',
        ]

        negative_prompt = ',(bad hands, bad anatomy, bad body, bad face, bad teeth, bad arms, bad legs, deformities:1.3),poorly drawn,deformed hands,deformed fingers,deformed faces,deformed eyes,mutated fingers,deformedbody parts,mutated body parts,mutated hands, disfigured,oversaturated,bad anatom,cropped, worst quality, low quality, jpeg artifacts, ugly, duplicate, morbid, mutilated, out of frame, extra fingers, mutated hands, poorly drawn hands, poorly drawn face, mutation, deformed, blurry, dehydrated, bad anatomy, bad proportions, extra limbs, cloned face, disfigured, gross proportions, deformed limbs, missing arms, missing legs, extra arms, extra legs, fused fingers, too many fingers, long neck,deformed eyes'

        #token = '03AFcWeA7BkorgG3DLH8M7Kc7n_OR4_aIzjM29WDFbTTGug037ubLfdcCo6CsxtvEaJH16JaVpr2IS0k3QOvEYrJdfp4EISFt7la0vnb1DOfD6KYHvW7udVmBu6dv5b-7sCJ3LDln6dhf63mFD_x7NAdXiAogdNgimfW2hwkXBwNGyFGD-p2C-VILC1BcEa95EjXHlWwB4FhZyKQtJAAkov1hAiETLrUZ85TDYDjSfiCziimtXpDCzW2fZzjZW0p_4VtBNdDkbNLwZC2ixM-_XuxpXoInQ1GXe3iqr9mWnILqloRRl_tblhFUO613WLEKwLZxnpBSITOHrg_TmIGqaIpjpv8koUYU0rgimOb1QXdEiaMyStZv8FkNggfOs4oDv895pJ3cUNKn8rk25htgDdDaa2ij6vCgX7OwdrWqSoMKR9uhgwDnuyR-228uvx5FMeRCzana6QAAe9cYgR7HfmR0B5M2l8qSyroWfnVD0HtnHoaqj71NLWht4VmuzhM8MfiZof5zDouxoWEFXl10HMj8S4yuGOCJ_U2y63BB5n4r3MmKWWlXxxCD-hS_zLqirr2lXK-SdVdEZFdDKVTMT4OKKpy4gVla0fJXtUoSdr_XpS0XYA6pwqVx4CezppNnHwHS5CHAoa2PMDaZExKZxeY5ArP2A8UzDHE9l2zclNckEtligkofBCnF-z9lmcGsMLco6sS-CHsNOT60UF4IIzTQQ71H7HArQbJd2WsJoCncCiYdfOzBrQF3PuVUzFG1OZPYvirteabptInbCkg1Rd0jXzuCZi1smo2HTpZ6cCC3V_uKn8mdQrlRNo7BLHm0s16Wp_Y16I_m2znBdEGM3KDu378uaMWeGIxZ-a-pq69bmGynGi_ZBu-LZnVSpCZ3ZMXawMr0ilOijnjKD8FvZWy_KAj9gfGfGoQ8_grlsLNzEFMtqHgPSTzeYwX5URqs16y71ydYJtxVqI3uszuEZbFlpee5Sa1nfOd48RVA9XRsWsED1lEvQTyGd7oXSlXlDl0uGEcbaMGoGFU9QlbZTpn0CuxF-3rJsZWRw0c3lbGktM-3bOu76T06c1lT530uSFC9HCmxNRrXnXWi26ynAnaiRmDwXsfyHKzi8wA1TwR1no233dL06TwWeBtzrfvUCkc-4aAP41R3yUK6uGA4QCjr38_5b8frgkVwEu6ecMYd-DVQVFLpOQB1gVkVAR9GZPWEz8XTes-hhDCoMNloCV9Bhb3L26m5EizMh0CLMM5Efvdl0Yf4j4QSgfTEnuaHfvM24HEdGSFcCXgg6hYCfLyejEAR5hW1OXqQ6_WGBNTRosJ6XIeQv2mPaOCWjWPKwE5_JkjhQZr6wZVDF5Yg2fBV-uma0Oq2BeLUcma92xicr7-5kbChI1eNneYHBruAkEBiqW2ZTPjuiaGQEL8WLOaWWefPf7Uvs4xJVvBGi5sJQUyY_jDCXS__Kcvcm0ZaAIgct4AanL2Gvr_URHhWzJiPYwd0J_jy3M8rYsljgGkD8Yd929z3bSwwxjkrYnUwW76FcXM3wZsNOHbiEqx1uuwNDrmdnXdviNdqRyyLzLI0EKqDXaZLXaxgM2_fFJ-lZeHeDeeoDQ1jqIchCZSJPSeNL_rTTy8r8PtCRBrpahK9VgdqtIdXyL0zDvP2bveQIZHM9BmYcpvsMD5QK092gKaVCYqnAwzwDSSI0QFU7nSUFH6FlLGabj7fh8XEi9iflmid8C6SF4CS6'

        token = self.get_recaptcha_token()

        json_data = {
            'prompt': prompt,
            'nprompt': negative_prompt,
            'steps': 20,
            'guidenceScale': 7,
            'style': 'PHOTOGRAPHY',
            'width': 1024,
            'height': 576,
            'alchemy': True,
            'pr': True,
            'token': token,
        }

        # This could be slow, and thats why I use 500 of timeout
        response = requests.post('https://fumes-api.onrender.com/leonardo', headers = HEADERS, json = json_data, timeout = 500)

        # This returns
        #localStorage.setItem("cookie", data.token);
        #localStorage.setItem("sub", data.sub);
        #localStorage.setItem("u", data.u);

        print(response.content)

        # We obtain the first image, but we have one in ['1'] and another one in ['2']
        image_url = response.json()['img']['1']
        download_image(image_url, output_filename)

    def __request_creation(self, request_url, prompt, negative_prompt):
        """
        Makes a creation request through the provided 'request_url' that is related to an
        specific image generation model.

        This method returns the 'id' that is the one with which we can ask for the status
        and receive the image to download when ready.

        This works here https://huggingface.co/spaces/FumesAI/text2Image-Models-Demo.
        """
        #negative_prompt = ' ,(bad hands, bad anatomy, bad body, bad face, bad teeth, bad arms, bad legs, deformities:1.3),poorly drawn,deformed hands,deformed fingers,deformed faces,deformed eyes,mutated fingers,deformedbody parts,mutated body parts,mutated hands, disfigured,oversaturated,bad anatom,cropped, worst quality, low quality, jpeg artifacts, ugly, duplicate, morbid, mutilated, out of frame, extra fingers, mutated hands, poorly drawn hands, poorly drawn face, mutation, deformed, blurry, dehydrated, bad anatomy, bad proportions, extra limbs, cloned face, disfigured, gross proportions, deformed limbs, missing arms, missing legs, extra arms, extra legs, fused fingers, too many fingers, long neck,deformed eyes'

        json_data = {
            'prompt': prompt,
            'negative_prompt': negative_prompt,
            'steps': '35',
            'gd': '7',
        }

        response = requests.post(request_url, headers = self.HUGGIN_FACE_IMAGE_MODELS_DEMO_HEADERS, json = json_data)
        
        id = response.json()

        return id
    
    def __request_status(self, id, tries = 12):
        """
        Makes a status request a maximum of 'tries' times to get the image when ready. It
        waits 10 seconds between each request.

        This method returns the image url if successfully generated, or False if something
        went wrong.

        This works here https://huggingface.co/spaces/FumesAI/text2Image-Models-Demo.
        """
        json_data = {
            'id': id,
        }

        for i in range(tries):
            time.sleep(10)
            response = requests.post('https://flask-hello-world-murex-sigma.vercel.app/status', headers = self.HUGGIN_FACE_IMAGE_MODELS_DEMO_HEADERS, json = json_data)

            response_json = response.json()
            if not 'status' in response_json and 'a' in response_json and 'b' in response_json:
                # It is returning 'a' and 'b' but are the same picture, don't know why
                image_url = response_json['a']

                return image_url
            
        return False
    
    def generate_sdxl(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/predict'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False

    def generate_playground_2_5(self, prompt, negative_prompt = '', output_filename = ''):
        """
        TODO: This method is not working
        """
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/piss'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_juggernaut_xl_v9(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/nut'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_proteus_04(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/p4'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_proteus_02(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/proteus'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_open_dalle_1_1(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/predict'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_proteus_03(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/p3'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_kandinsky_2_2(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/k'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_sdxl_emoji(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/emo'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_wuerstchen(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/w'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_dreamshaper_xl_turbo(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/dream'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
    
    def generate_pixart_alpha(self, prompt, negative_prompt = '', output_filename = ''):
        URL = 'https://flask-hello-world-murex-sigma.vercel.app/pixart'
        id = self.__request_creation(URL, prompt, negative_prompt)
        image_url = self.__request_status(id)

        if image_url:
            download_image(image_url, output_filename)
            return True

        # Raise exception
        return False
        




    def test(self, prompt):
        # From here: https://dream.ai/create
        # TODO: This is not from AIFumes but I'm testing it quickly
        HEADERS = {
            'authority': 'paint.api.wombo.ai',
            'accept': '*/*',
            'accept-language': 'es-ES,es;q=0.9',
            'authorization': 'bearer eyJhbGciOiJSUzI1NiIsImtpZCI6IjYwOWY4ZTMzN2ZjNzg1NTE0ZTExMGM2ZDg0N2Y0M2M3NDM1M2U0YWYiLCJ0eXAiOiJKV1QifQ.eyJwcm92aWRlcl9pZCI6ImFub255bW91cyIsImlzcyI6Imh0dHBzOi8vc2VjdXJldG9rZW4uZ29vZ2xlLmNvbS9wYWludC1wcm9kIiwiYXVkIjoicGFpbnQtcHJvZCIsImF1dGhfdGltZSI6MTcxMDI2OTIyMSwidXNlcl9pZCI6IkNoUGc1TTdsSWtOUUlyVHpLOHJUZzlCNDVZMzIiLCJzdWIiOiJDaFBnNU03bElrTlFJclR6SzhyVGc5QjQ1WTMyIiwiaWF0IjoxNzEwMjY5MjIxLCJleHAiOjE3MTAyNzI4MjEsImZpcmViYXNlIjp7ImlkZW50aXRpZXMiOnt9LCJzaWduX2luX3Byb3ZpZGVyIjoiYW5vbnltb3VzIn19.eOsyYTm6ZrCunqah3zRuejVot4gYxEeTGkm9mvghSvuiHTXJqnCUduSe0huqBCUFhXIwvwrxM88A_BZlQNI1eKK8RXFcDFgo1B3hmaPvBvM7T06y-OIlTSUDlLsv5E0oIdd7POeKuxHJVWdxnfynF1-y3upDb37EBywZbIhAOXJWibsha8WNy4NDPDbmSG0b5WAiUCCPTr4eAetGaV7ouBssn1WdAfAsasbK0Ys5vxjXSlNMSWVDFWu7sNlDys6nnNGRgQlOKqheVzBTusBvfts8JnuKGyYp_kSEuadnybNPy418P0ySNbfGgKBw4kSkKDdU_wS6pDUdFl6q2Ju8VQ',
            'content-type': 'text/plain;charset=UTF-8',
            'origin': 'https://dream.ai',
            'referer': 'https://dream.ai/',
            'sec-ch-ua': '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'cross-site',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36',
            'x-app-version': 'WEB-2.0.0',
            'x-validation-token': 'eyJraWQiOiJYcEhKU0EiLCJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJzdWIiOiIxOjE4MTY4MTU2OTM1OTp3ZWI6Mjc3MTMzYjU3ZmVjZjU3YWYwZjQzYSIsImF1ZCI6WyJwcm9qZWN0c1wvMTgxNjgxNTY5MzU5IiwicHJvamVjdHNcL3BhaW50LXByb2QiXSwicHJvdmlkZXIiOiJyZWNhcHRjaGFfZW50ZXJwcmlzZSIsImlzcyI6Imh0dHBzOlwvXC9maXJlYmFzZWFwcGNoZWNrLmdvb2dsZWFwaXMuY29tXC8xODE2ODE1NjkzNTkiLCJleHAiOjE3MTAyODcyMjAsImlhdCI6MTcxMDI2OTIyMCwianRpIjoiYXE2MXFWNDBaa3dRaFI1Q1E3U0xTcDI5cWtXSnhiOFFNS2RocmZ3Yk5rNCJ9.je_jITU4LhA4UlFwkJp0Pke8Pv6VB2ceLidBuwAbPoPWLO2MocVPpi-sBzukNQfOLmOytOZnzzSgon11kU8yP2vE3yJnzVlIelzN79BYIs7IIiDjftlqhtwZNLY-Est3VljRf7WE_p5FuW445PWGqgj-te_N-XFlKrjAReGbED8UBIpF3ZeQDwrRB9vCT4RxuxqiGltQ-vW3W4H-b58VWSizDiAz0gn5yj6132fZhavA-uPRgwCwkZyHhJGoJmXt-O98PB6ArQ8yNvnYpt3Z2VHxJdlqsy8g3UXAMbUS-wPjM0RchvtAsSe2lB4rCewtJKzsji31uFSQf6_9qNnxud9s-6Yt2d6TcMoP5dEAM_fr2iW3aht_LJ7WBBWHhNplYHUKN4b3uVQneaT_v_PAY70xMachEnrj0vYfDo-o-KPaWWxfs2oDpAGhOlERjejDNRkSfYfwEzoI-AhxYUbnWDCQNmw8Lp_7sJ8JJHr7D3RwbnvXRLTYIQYNoDldbpA6',
        }

        data = '{"is_premium":false,"input_spec":{"prompt":"' + prompt + '","style":115,"display_freq":10}}'

        response = requests.post('https://paint.api.wombo.ai/api/v2/tasks', headers = HEADERS, data = data)
        # I think that when token expired, this won't be a json
        response_json = response.json()
        id = response_json['id']
        # 'id' and 'user_id'?

        STATUS_HEADERS = {
            'authority': 'paint.api.wombo.ai',
            'accept': '*/*',
            'accept-language': 'es-ES,es;q=0.9',
            'authorization': 'bearer eyJhbGciOiJSUzI1NiIsImtpZCI6IjYwOWY4ZTMzN2ZjNzg1NTE0ZTExMGM2ZDg0N2Y0M2M3NDM1M2U0YWYiLCJ0eXAiOiJKV1QifQ.eyJwcm92aWRlcl9pZCI6ImFub255bW91cyIsImlzcyI6Imh0dHBzOi8vc2VjdXJldG9rZW4uZ29vZ2xlLmNvbS9wYWludC1wcm9kIiwiYXVkIjoicGFpbnQtcHJvZCIsImF1dGhfdGltZSI6MTcxMDI2OTIyMSwidXNlcl9pZCI6IkNoUGc1TTdsSWtOUUlyVHpLOHJUZzlCNDVZMzIiLCJzdWIiOiJDaFBnNU03bElrTlFJclR6SzhyVGc5QjQ1WTMyIiwiaWF0IjoxNzEwMjY5MjIxLCJleHAiOjE3MTAyNzI4MjEsImZpcmViYXNlIjp7ImlkZW50aXRpZXMiOnt9LCJzaWduX2luX3Byb3ZpZGVyIjoiYW5vbnltb3VzIn19.eOsyYTm6ZrCunqah3zRuejVot4gYxEeTGkm9mvghSvuiHTXJqnCUduSe0huqBCUFhXIwvwrxM88A_BZlQNI1eKK8RXFcDFgo1B3hmaPvBvM7T06y-OIlTSUDlLsv5E0oIdd7POeKuxHJVWdxnfynF1-y3upDb37EBywZbIhAOXJWibsha8WNy4NDPDbmSG0b5WAiUCCPTr4eAetGaV7ouBssn1WdAfAsasbK0Ys5vxjXSlNMSWVDFWu7sNlDys6nnNGRgQlOKqheVzBTusBvfts8JnuKGyYp_kSEuadnybNPy418P0ySNbfGgKBw4kSkKDdU_wS6pDUdFl6q2Ju8VQ',
            'origin': 'https://dream.ai',
            'referer': 'https://dream.ai/',
            'sec-ch-ua': '"Chromium";v="122", "Not(A:Brand";v="24", "Google Chrome";v="122"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'cross-site',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36',
            'x-app-version': 'WEB-2.0.0',
        }

        # 'Web Dream API KEY' z8w6OPCOBstepOZXbR7L

        tries = 12
        for i in range(tries):
            time.sleep(10)
            response = requests.get('https://paint.api.wombo.ai/api/v2/tasks/' + id, headers = STATUS_HEADERS)
            response_json = response.json()

            if 'result' in response_json and response_json['result'] and response_json['result']['final']:
                download_by_url(response_json['result']['final'], 'test_dreamai.png')
                return True
            
        return False


# TODO: Maybe this is not the best place to put this module but
# here it is. Fell free to move it.

def remove_noise_from_audio_file(audio_filename: str, audio_output_filename: str):
    """
    Removes the noise from the provided 'audio_filename' and creates a new
    file 'audio_output_filename' without noise.

    # TODO: This fails when .mp3 is used, so we need to transform into wav.

    # TODO: Output file must be also wav

    # TODO: What about audioclip instead of audiofile? Is it possible? (?)
    """
    # Based on this (https://medium.com/@devesh_kumar/how-to-remove-noise-from-audio-in-less-than-10-seconds-8a1b31a5143a)
    # https://github.com/Rikorose/DeepFilterNet
    # TODO: This is failing now saying 'File contains data in an unknon format'...
    # I don't know if maybe some library, sh*t...
    # Load default model
    from df.enhance import enhance, init_df, load_audio, save_audio
    from yta_general_utils.audio_processor import mp3_to_wav
    from yta_general_utils.file_processor import delete_file

    TMP_WAV_FILENAME = 'tmp_wav.wav'
    if audio_filename.endswith('.mp3'):
        # TODO: Maybe it is .wav but not that format...
        mp3_to_wav(audio_filename, TMP_WAV_FILENAME)
        audio_filename = TMP_WAV_FILENAME

    model, df_state, _ = init_df()
    audio, _ = load_audio(audio_filename, sr = df_state.sr())
    # Remove the noise
    enhanced = enhance(model, df_state, audio)

    save_audio(audio_output_filename, enhanced, df_state.sr())

    try:
        delete_file(TMP_WAV_FILENAME)
    except:
        pass


from moviepy.editor import AudioFileClip, CompositeAudioClip, concatenate_audioclips
from yta_general_utils.file_processor import get_current_file_abspath

RESOURCES_ABSOLUTE_PATH = get_current_file_abspath(3) + 'resources/audio/sound/'

def create_silence_audio_file(duration, output_filename):
    """
    Creates a silence audio file that lasts the 'duration' provided in 
    seconds.
    """
    create_silence_audio_clip(duration).write_audiofile(output_filename)

def create_silence_audio_clip(duration) -> AudioFileClip:
    """
    This method creates a silence AudioFileClip of the provided 'duration' in
    seconds and returns it.
    """
    SILENT_AUDIO_FILENAME = RESOURCES_ABSOLUTE_PATH + 'silent.wav'

    return AudioFileClip(SILENT_AUDIO_FILENAME).subclip(0, duration)

def create_typing_audio_file(output_filename: str):
    """
    This method creates a typing audio of 3.5 seconds (3s of typing, 0.5 of silence).
    """
    if not output_filename:
        return None
    
    create_typing_audio_clip().write_audiofile(output_filename)

def create_typing_audio_clip() -> CompositeAudioClip:
    """
    This method creates a typing audio of 3.5 seconds (3s of typing, 0.5 of silence)
    and returns it as a clip.
    """
    AUDIO_TYPING_3S_FILENAME = RESOURCES_ABSOLUTE_PATH + 'typing_3s.mp3'

    typing_audio_clip = AudioFileClip(AUDIO_TYPING_3S_FILENAME)
    silence_audio_clip = create_silence_audio_clip(0.5)

    return concatenate_audioclips([typing_audio_clip, silence_audio_clip])



from pydub import AudioSegment, silence

# TODO: Maybe rename to 'silence.py' in singular (?)

def detect_silences_in_audio_file(audio_filename: str, silence_ms_min_length: int = 250):
    """
    This method detects the silences existing in the provided 'audio_filename' that
    are at least 'silence_ms_min_length' long (this time is in milliseconsd). It 
    will return an array containing tuples (X.XX, Y.YY) of start and end silence 
    moments (in seconds).
    """
    # TODO: Getting AudioSegment from AudioFileClip and not files would be awesome
    audio = __get_audio_segment(audio_filename)

    dBFS = audio.dBFS
    silences = silence.detect_silence(audio, min_silence_len = silence_ms_min_length, silence_thresh = dBFS - 16)

    # [(1.531, 1.946), (..., ...), ...] in seconds
    return [((start / 1000), (stop / 1000)) for start, stop in silences]

# TODO: These below should not be here maybe (?)
# About sound intensity
def test_sound_intensity(audio_filename):
    audio: AudioSegment = __get_audio_segment(audio_filename)
    # TODO: Do more stuff 

def __get_audio_segment(audio_filename: str) -> AudioSegment:
    """
    Reads the provided 'audio_filename' and returns it as a
    'pydub' library AudioSegment object.
    """
    from yta_general_utils.file_processor import file_is_audio_file, get_file_extension
    
    if not audio_filename:
        return None
    
    if not file_is_audio_file(audio_filename):
        return None
    
    
    myaudio = AudioSegment.from_file(audio_filename)

    """
    # TODO: This is really needed? I need to do more tests
    extension = get_file_extension(audio_filename)
    if extension == 'mp3':
        myaudio = AudioSegment.from_mp3(audio_filename)
    elif extension == 'wav':
        myaudio = AudioSegment.from_wav(audio_filename)
    else:
        # TODO: Do this work? I think no, because it is not configured
        myaudio = AudioSegment.from_file(audio_filename)
    """

    return myaudio





def image_file_to_gameboy(image_filename: str, output_filename: str):
    """
    Receives an 'image_filename' and turns it into a game boy
    art (turning it into the game boy screen pixel colors) and
    saves it as 'output_filename'.

    The input file image will be resized to (480, 432), that is
    the game boy screen size.

    # TODO: Make this method work within a video (maybe with
    pygame) and set the code in video.edition section
    """
    if not image_filename:
        return None
    
    if not output_filename:
        return None
    
    import cv2
    import scipy.spatial as sp

    # These below are the colors used in GB
    GAMEBOY_COLORS = [
        (155, 188, 15),
        (139, 172, 15),
        (48, 98, 48),
        (15, 56, 15),
        (15, 56, 15)
    ]

    # Original gameboy screen size is (480, 432)
    image = cv2.imread(image_filename)
    image = cv2.resize(image, (480, 432)) 
    rows, cols, _ = image.shape
    new_image = image.copy()

    """
    TODO: Improve this method. 
    
    Scroll the whole image, but do one step first. Check
    in a list if the pixel is written. If not, look for
    the nearest rgb color and store it in that list. Then
    write it in the new image. If found, you don't need 
    to look for the nearest (through tree.query) again.
    You just need to look for it in the list.

    Or maybe that doesn't make it faster. Don't know.
    """

    tree = sp.KDTree(GAMEBOY_COLORS)
    for i in range(rows):
        for j in range(cols):
            pixel_color = image[i, j]
            distance, result = tree.query(pixel_color) 
            nearest_color = GAMEBOY_COLORS[result]
            new_image[i, j] = nearest_color

    cv2.imwrite(output_filename, new_image)
    print(output_filename + ' written successfully.')



from yta_general_utils.file_downloader import download_audio
import requests

# TODO: Check this because I don't know if this webpage is using the tts (coqui)
# library as the generator engine. If that, I have this engine in 'coqui.py' file
# so I don't need this (that is not stable because is based in http requests)
def narrate_tts3(text, output_filename):
    """
    Aparrently not limited. Check, because it has time breaks and that stuff.
    """
    # From here: https://ttsmp3.com/
    headers = {
        'accept': '*/*',
        'accept-language': 'es-ES,es;q=0.9',
        'content-type': 'application/x-www-form-urlencoded',
        'origin': 'https://ttsmp3.com',
        'referer': 'https://ttsmp3.com/',
        'sec-ch-ua': '"Google Chrome";v="123", "Not:A-Brand";v="8", "Chromium";v="123"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"Windows"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
    }

    VOICES = ['Lupe', 'Penelope', 'Miguel']
    data = {
        'msg': text,
        'lang': 'Lupe',
        'source': 'ttsmp3',
    }

    response = requests.post('https://ttsmp3.com/makemp3_new.php', headers = headers, data = data)
    response = response.json()
    url = response['URL']
    # "https://ttsmp3.com/created_mp3/8b38a5f2d4664e98c9757eb6db93b914.mp3"
    download_audio(url, output_filename)


from yta_general_utils.tmp_processor import create_tmp_filename
from moviepy.editor import VideoFileClip, ImageClip, CompositeVideoClip, vfx
from PIL import Image, ImageDraw, ImageFont

class CustomGreenScreenImage:
    """
    This class is used to build custom green screen that has specific green screen positions and
    title and description font sizes.
    """
    def __init__(self, filename: str, rgb_color: tuple, ulx: int, uly: int, drx: int, dry: int, title: str, title_font: ImageFont.FreeTypeFont, title_color: str, title_x: int, title_y: int, description: str, description_font: ImageFont.FreeTypeFont, description_color: str, description_x: int, description_y: int):
        # TODO: What about custom values ?
        self.__filename = filename
        self.ulx = ulx
        self.uly = uly
        self.drx = drx
        self.dry = dry
        self.rgb_color = rgb_color
        self.__title = title
        self.__title_font = title_font
        self.__title_color = title_color
        self.__title_x = title_x
        self.__title_y = title_y
        self.__description = description
        self.__description_font = description_font
        self.__description_color = description_color
        self.__description_x = description_x
        self.__description_y = description_y

    def save(self, output_filename):
        base = Image.open(self.__filename)
        draw = ImageDraw.Draw(base)   

        # We need to write title if existing
        if self.__title:
            title_position = (self.__title_x, self.__title_y)
            draw.text(title_position, self.__title, font = self.__title_font, fill = self.__title_color)

        if self.__description:
            description_position = (self.__description_x, self.__description_y)
            draw.text(description_position, self.__description, font = self.__description_font, fill = self.__description_color)

        # Anything else to handle?

        base.save(output_filename, quality = 100)

    def __generate_green_screen(self):
        pass

    def __add_clip(self, clip):
        """
        Adds the received 'clip' to the green screen image and displays it inside the green
        screen area.
        """
        # We write title and description if needed
        tmp_filename = create_tmp_filename('tmp_gs.png')
        self.save(tmp_filename)

        green_screen_clip = ImageClip(tmp_filename, duration = clip.duration).fx(vfx.mask_color, color = self.rgb_color, thr = 100, s = 5)

        width = self.drx - self.ulx
        clip = clip.resize(width = width).set_position((self.ulx, self.uly))

        return CompositeVideoClip([clip, green_screen_clip], size = green_screen_clip.size)

    def insert_video(self, video_filename, output_filename):
        """
        Inserts the provided 'video_filename' in the green screen and generates a new
        video which is stored locally as 'output_filename'.
        """
        clip = VideoFileClip(video_filename)
        final_clip = self.__add_clip(clip)
        final_clip.write_videofile(output_filename, fps = clip.fps)

    def from_clip(self, clip):
        """
        Receives a 'clip' and generates a new one that is the provided one inside of the
        green screen.
        """
        return self.__add_clip(clip)

    def insert_image(self, image_filename, output_filename):
        """
        Inserts the provided 'image_filename' in the green screen and generates a new 
        image wich is stored locally as 'output_filename'.
        """
        # I do the trick with moviepy that is working for videos...
        clip = ImageClip(image_filename, duration = 1 / 60)
        final_clip = self.__add_clip(clip)
        final_clip.save_frame(output_filename, t = 0)



from PIL import Image

import numpy as np
import skimage.exposure
import cv2

def get_green_screen_position(image_filename):
    """
    Detects the green screen color of the provided 'image_filename' and then looks for
    the upper left corner and the bottom right corner.

    This method return an object containing 'ulx', 'uly', 'drx', 'dry' coords. It also
    returns the 'rgb_color' most common green color as an (r, g, b).

    This will return None in ['rgb_color'] field if no green color detected.

    # TODO: Maybe rename to 'get_green_screen_properties' (?)
    """
    green_rgb_color = get_most_common_rgb_color(image_filename)

    image = Image.open(image_filename).convert('RGB')

    upper_left = {
        'x': 99999,
        'y': 99999,
    }
    down_right = {
        'x': -1,
        'y': -1,
    }

    for x in range(image.width):
        for y in range(image.height):
            rgb_color = (r, g, b) = image.getpixel((x, y))

            if rgb_color == green_rgb_color:
                if x < upper_left['x']:
                    upper_left['x'] = x
                if y < upper_left['y']:
                    upper_left['y'] = y
                
                """
                if x <= upper_left['x'] and y <= upper_left['y']:
                    upper_left = {
                        'x': x,
                        'y': y,
                    }
                """

                if x > down_right['x']:
                    down_right['x'] = x
                if y > down_right['y']:
                    down_right['y'] = y

                """
                if x >= down_right['x'] and y >= down_right['y']:
                    down_right = {
                        'x': x,
                        'y': y,
                    }
                """

    # We apply some margin to make sure we fit the green screen
    MARGIN = 2

    if (upper_left['x'] - MARGIN) > 0:
        upper_left['x'] -= MARGIN
    else:
        upper_left['x'] = 0

    if (upper_left['y'] - MARGIN) > 0:
        upper_left['y'] -= MARGIN
    else:
        upper_left['y'] = 0

    if (down_right['x'] + MARGIN) < 1920:
        down_right['x'] += MARGIN
    else:
        down_right['x'] = 1920

    if (down_right['y'] + MARGIN) < 1080:
        down_right['y'] += MARGIN
    else:
        down_right['y'] = 1080

    return {
        'rgb_color': green_rgb_color,
        'ulx': upper_left['x'],
        'uly': upper_left['y'],
        'drx': down_right['x'],
        'dry': down_right['y'],
    }

def get_most_common_rgb_color(image_filename, force_green = True):
    """
    Returns the most common green rgb color that exist in the provided
    'image_filename'. There could be no green color so it will return
    None, or the green color as (r, g, b) if existing.

    # TODO: This method could be made more generic by using a range
    # and then specializing this one by using that with green range
    """
    colors = {}
    image = Image.open(image_filename).convert('RGB')

    # We will check the most common rgb color (should be the green of mask)
    for x in range(image.width):
        for y in range(image.height):
            rgb_color = (r, g, b) = image.getpixel((x, y))

            if not rgb_color in colors:
                colors[rgb_color] = 1
            else:
                colors[rgb_color] += 1

    # Check which one is the most common
    most_used_rgb_color = {
        'color': None,
        'times': 0,
    }
    for key, value in colors.items():
        if force_green:
            # We only care about green colors
            r, g, b = key
            is_green = (r >= 0 and r <= 100) and (g >= 100 and g <= 255) and (b >= 0 and b <= 100)
            if is_green:
                if value > most_used_rgb_color['times']:
                    most_used_rgb_color = {
                        'color': key,
                        'times': value
                    }
        else:
            if value > most_used_rgb_color['times']:
                most_used_rgb_color = {
                    'color': key,
                    'times': value
                }

    return most_used_rgb_color['color']

def remove_green_screen_from_image_file(image_filename: str, output_filename: str):
    """
    Removes the green screen in the provided 'image_filename' (if
    existing) and creates a new image as 'output_filename' with a
    transparent mask instead of the green screen.
    """
    # From https://stackoverflow.com/a/72280828
    if not image_filename:
        return None
    
    if not output_filename:
        return None

    # load image
    img = cv2.imread(image_filename)

    # convert to LAB
    lab = cv2.cvtColor(img, cv2.COLOR_BGR2LAB)

    # extract A channel
    A = lab[:,:,1]

    # threshold A channel
    thresh = cv2.threshold(A, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)[1]

    # blur threshold image
    blur = cv2.GaussianBlur(thresh, (0,0), sigmaX = 5, sigmaY = 5, borderType = cv2.BORDER_DEFAULT)

    # stretch so that 255 -> 255 and 127.5 -> 0
    mask = skimage.exposure.rescale_intensity(blur, in_range = (127.5, 255), out_range = (0, 255)).astype(np.uint8)

    # add mask to image as alpha channel
    result = img.copy()
    result = cv2.cvtColor(img,cv2.COLOR_BGR2BGRA)
    result[:,:,3] = mask

    # save output
    cv2.imwrite(output_filename, result)


from moviepy.editor import VideoFileClip
from math import floor
from yta_general_utils.tmp_processor import create_tmp_filename
from yta_general_utils.file_processor import rename_file, file_is_video_file
from yta_general_utils.type_checker import variable_is_type
from typing import Union

def rescale_video(video_input: Union[VideoFileClip, str], output_width: int = 1920, output_height: int = 1080, output_filename: Union[str, None] = None):
    """
    This method was created to rescale videos upper to 1920x1080 or 1080x1920. This is,
    when a 4k video appears, we simplify it to 1080p resolution to work with only that
    resolution. This method returns the VideoFileClip rescaled but also writes it if
    'output_filename' provided.

    The 'output_width' and 'output_height' variables must be [1920 and 1080] or [1080 and
    1920]. Any other pair is not valid.

    This method is used in the script-guided video generation. Please, do not touch =).

    TODO: This method is very strict, so it will need a revision
    """
    # We only want to accept 16/9 or 9/16 by now, so:
    if not (output_width == 1920 and output_height == 1080) and not (output_width == 1080 and output_height == 1920):
        print('Sorry, not valid input parameters.')
        return None
    
    if not video_input:
        return None
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)
    
    SCALE_WIDTH = 16
    SCALE_HEIGHT = 9
    if output_width == 1080 and output_height == 1920:
        SCALE_WIDTH = 9
        SCALE_HEIGHT = 16

    width = video_input.w
    height = video_input.h

    # We avoid things like 1927 instead of 1920
    new_width = width - width % SCALE_WIDTH
    new_height = height - height % SCALE_HEIGHT

    proportion = new_width / new_height

    if proportion > (SCALE_WIDTH / SCALE_HEIGHT):
        print('This video has more width than expected. Cropping horizontally.')
        while (new_width / new_height) != (SCALE_WIDTH / SCALE_HEIGHT):
            new_width -= SCALE_WIDTH
    elif proportion < (SCALE_WIDTH / SCALE_HEIGHT):
        print('This video has more height than expected. Cropping vertically.')
        while (new_width / new_height) != (SCALE_WIDTH / SCALE_HEIGHT):
            new_height -= SCALE_HEIGHT

    print('Final: W' + str(new_width) + ' H' + str(new_height))
    videoclip_rescaled = video_input.crop(x_center = floor(width / 2), y_center = floor(height / 2), width = new_width, height = new_height)
    
    # Force output dimensions
    if new_width != output_width:
        print('Forcing W' + str(output_width) + ' H' + str(output_height))
        videoclip_rescaled = videoclip_rescaled.resize(width = output_width, height = output_height)

    if output_filename:
        # TODO: Check extension
        tmp_video_filename = create_tmp_filename('scaled.mp4')
        tmp_audio_filename = create_tmp_filename('temp-audio.m4a')
        videoclip_rescaled.write_videofile(tmp_video_filename, codec = 'libx264', audio_codec = 'aac', temp_audiofile = tmp_audio_filename, remove_temp = True)
        rename_file(tmp_video_filename, output_filename, True)

    return videoclip_rescaled


from gtts import gTTS
from enum import Enum
class TLD(Enum):
    SPANISH_MEXICO = 'com.mx'
    SPANISH_SPAIN = 'es'
    SPANISH_US = 'us'

class Language(Enum):
    SPANISH = 'es'

def narrate(text,  language: Language = Language.SPANISH, tld: TLD = TLD.SPANISH_SPAIN, output_filename = None):
    """
    Creates an audio narration of the provided 'text' with the Google voice and stores it
    as 'output_filename'. This will use the provided 'language' language for the narration.
    """
    if not output_filename:
        return None
    # TODO: Check that 'language' and 'tld' are valid
    
    language = language.value
    tld = tld.value
    
    # TODO: Check valid language tag in this table (https://en.wikipedia.org/wiki/IETF_language_tag)
    # TODO: Use this library for languages (https://pypi.org/project/langcodes/)
    # TODO: Here we have the languages and tlds (https://gtts.readthedocs.io/en/latest/module.html#languages-gtts-lang)
    tts = gTTS(text, lang = language, tld = tld)
    tts.save(output_filename)



from PIL import Image

import requests
import os

class HugginFace:
    API_KEY = os.environ.get("HUGGINFACE_API_KEY")
    HEADERS = {"Authorization": f"Bearer {API_KEY}"}
    
    def __init__(self, api_url):
        self.api_url = api_url

    def __request(self, payload):
        response = requests.post(self.api_url, headers = self.HEADERS, json = payload)
        print(response.content)
        return response

    def request_image(self, payload):
        return self.__request(payload).content

    def request_json(self, payload):
        return self.__request(payload).json()
    
    def download_image(self, prompt, output_filename):
        """
        Makes a request with the provided payload and stores the image received as response
        in the local storage.

        This method must be used with image generation urls.
        """
        image_content = self.request_image(prompt)
        with open(output_filename, 'wb') as outfile:
            outfile.write(image_content)













# from PIL import Image

# import requests
# import os
# import io

# API_KEY = os.environ.get("HUGGINFACE_API_KEY")
# HEADERS = {"Authorization": f"Bearer {API_KEY}"}

# """
# You go to any existing model in HugginFace platform, then click on 'Deploy' (on
# the right side) and 'Inference API' and there you have the url and the query
# structure.
# """

# def __request(api_url, payload):
#     response = requests.post(api_url, headers = HEADERS, json = payload)
#     print(response)
#     return response

# def __request_image(api_url, payload):
#     return __request(api_url, payload).content

# def __request_json(api_url, payload):
#     return __request(api_url, payload).json()

# def test_image():
#     #API_URL = 'https://api-inference.huggingface.co/models/stabilityai/stable-cascade'
#     #API_URL = 'https://api-inference.huggingface.co/models/stabilityai/stable-cascade-prior'
#     #API_URL = 'https://api-inference.huggingface.co/models/runwayml/stable-diffusion-v1-5'
#     #API_URL = 'https://api-inference.huggingface.co/models/stabilityai/stable-diffusion-2-1'
#     API_URL = 'https://api-inference.huggingface.co/models/prompthero/openjourney'

#     PAYLOAD = {
#         'inputs': 'Hiperrealistic monk between two cars in the middle of a city'
#     }

#     response = __request_image(API_URL, PAYLOAD)
#     Image.open(io.BytesIO(response)).save('test_tintin.png')

# def test_tintin():
#     API_URL = 'https://api-inference.huggingface.co/models/Pclanglais/TintinIA'

#     PAYLOAD = {
#         'inputs': 'Astronaut riding a horse and holding a coffee'
#     }

#     response = __request_image(API_URL, PAYLOAD)
#     Image.open(io.BytesIO(response)).save('test_tintin.png')

# def test_gemma():
#     API_URL = 'https://api-inference.huggingface.co/models/google/gemma-7b'

#     PAYLOAD = {
#         'inputs': '¿Cuántos habitantes tiene España?'
#     }

#     response = __request_json(API_URL, PAYLOAD)

#     print(response)

# def test():
#     API_URL = 'https://api-inference.huggingface.co/models/deepset/roberta-base-squad2'
    
#     PAYLOAD = {
#         'inputs': {
#             'question': 'What is my name?',
#             'context': 'My name is Clara and I live in Berkeley.',
#         }
#     }

#     response = __request_json(API_URL, PAYLOAD)
#     #response has, in this case, 'answer'
#     print(response)


from moviepy.editor import VideoFileClip, ImageClip, CompositeVideoClip, vfx, concatenate_videoclips
from PIL import ImageFont

class CustomGreenScreenVideo:
    """
    This class is used to build green screen videos by inserting videos in this green screen video.
    """
    def __init__(self, filename: str, rgb_color: tuple, ulx: int, uly: int, drx: int, dry: int, title: str, title_font: ImageFont.FreeTypeFont, title_color: str, title_x: int, title_y: int, description: str, description_font: ImageFont.FreeTypeFont, description_color: str, description_x: int, description_y: int):
        # TODO: What about custom values ?
        self.__filename = filename
        self.ulx = ulx
        self.uly = uly
        self.drx = drx
        self.dry = dry
        self.rgb_color = rgb_color
        # By now these elements are not used
        self.__title = title
        self.__title_font = title_font
        self.__title_color = title_color
        self.__title_x = title_x
        self.__title_y = title_y
        self.__description = description
        self.__description_font = description_font
        self.__description_color = description_color
        self.__description_x = description_x
        self.__description_y = description_y

    def __generate_green_screen(self):
        pass

    def __add_clip(self, clip):
        """
        Adds the received 'clip' to the green screen video and displays it inside the green
        screen area.
        """
        # TODO: Write title and description with textclips
        #green_screen_clip = VideoFileClip(self.__filename, duration = clip.duration).fx(vfx.mask_color, color = self.rgb_color, thr = 100, s = 5)
        green_screen_clip = VideoFileClip(self.__filename).fx(vfx.mask_color, color = self.rgb_color, thr = 100, s = 5)
        
        width = self.drx - self.ulx
        # If the provided clip is shorter than our green screen: easy, crop the green screen
        # If the provided clip is longer than our green screen: I use the green screen duration
        # and let the clip be original the rest of the time
        if green_screen_clip.duration > clip.duration:
            green_screen_clip = green_screen_clip.set_duration(clip.duration)
            # Clip will be displayed inside the green screen area
            clip = clip.resize(width = width).set_position((self.ulx, self.uly))
            clip = CompositeVideoClip([clip, green_screen_clip], size = green_screen_clip.size)
        elif clip.duration > green_screen_clip.duration:
            # First subclip will be displayed inside the green screen area
            first_clip = clip.subclip(0, green_screen_clip.duration).resize(width = width).set_position((self.ulx, self.uly))
            # Second clip will be as the original one
            second_clip = clip.subclip(green_screen_clip.duration, clip.duration)
            clip = concatenate_videoclips([
                CompositeVideoClip([first_clip, green_screen_clip], size = green_screen_clip.size),
                second_clip
            ])
        else:
            clip = CompositeVideoClip([clip, green_screen_clip], size = green_screen_clip.size)

        return clip
    
    def insert_video(self, video_filename, output_filename):
        """
        Inserts the provided 'video_filename' in the green screen and generates a new
        video which is stored locally as 'output_filename'.
        """
        clip = VideoFileClip(video_filename)
        final_clip = self.__add_clip(clip)
        final_clip.write_videofile(output_filename, fps = clip.fps)

    def from_clip(self, clip):
        """
        Receives a 'clip' and generates a new one that is the provided one inside of the
        green screen.
        """
        return self.__add_clip(clip)

    def insert_image(self, image_filename, output_filename):
        """
        Inserts the provided 'image_filename' in the green screen and generates a new 
        image wich is stored locally as 'output_filename'.
        """
        # I do the trick with moviepy that is working for videos...
        clip = ImageClip(image_filename, duration = 1 / 60)
        final_clip = self.__add_clip(clip)
        final_clip.save_frame(output_filename, t = 0)



from moviepy.editor import VideoFileClip
from yta_general_utils.type_checker import variable_is_type
from yta_general_utils.file_processor import file_is_video_file
from typing import Union

def extract_frames_from_video(video_input: Union[VideoFileClip, str], output_folder: str):
    """
    This method will extract all the 'video_input' frames in the
    provided 'output_folder' with the 'frameXXXXX.png' name, starting
    from 0 to the last frame.
    """
    if not video_input:
        return None
    
    if not output_folder:
        return None
    
    if not output_folder.endswith('/'):
        output_folder += '/'
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)

    video_input.write_images_sequence(output_folder + 'frame%05d.png')

    return output_folder


from image.edition.filter.pixelate import pixelate_image_file
from moviepy.editor import VideoFileClip, ImageSequenceClip
from yta_general_utils.tmp_processor import create_custom_tmp_filename
from yta_general_utils.file_processor import file_is_video_file
from yta_general_utils.type_checker import variable_is_type
from typing import Union

import shutil

def pixelate_video_file(video_input: Union[VideoFileClip, str], output_filename: str):
    """
    This methods pixelates the video progressively and then depixelates
    it progressively again. Finally, it writes the new video as
    'output_filename'.

    TODO: Review this to improve and to check, please, it maybe 
    should be experimental (?)
    """
    # TODO: There is another video pixelating method that I think is
    # faster than this one (the 'artistic.pixelated_video.py')
    if not video_input:
        return None
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)
    
    if not output_filename:
        return None
    
    # We extract frames, and pixelate in ascending order
    original_frames_array = []
    for frame in video_input.iter_frames():
        frame_name = create_custom_tmp_filename('tmp_frame_' + str(len(original_frames_array)) + '.png')
        original_frames_array.append(frame_name)
    video_input.write_images_sequence(create_custom_tmp_filename('tmp_frame_%01d.png', logger = 'bar'))

    # Remove green screen of each frame and store it
    processed_frames_array = []
    num_of_frames = round(video_input.fps * video_input.duration)
    # By now, from start to end
    start_frame = 0
    end_frame = num_of_frames
    pixelate_factor = (1024, 1024)
    pixelation_duration = int(1024 / 64) - 2
    # We will start pixelating, maintain, and remove pixelation
    for index, frame in enumerate(original_frames_array):
        tmp_frame_filename = create_custom_tmp_filename('tmp_frame_processed_' + str(index) + '.png')
        processed_frames_array.append(tmp_frame_filename)
        if index >= start_frame and index <= end_frame:
            pixelate_image_file(frame, pixelate_factor, tmp_frame_filename)

            if index <= (start_frame + pixelation_duration):
                # We are in incrementing process
                tmp_list = list(pixelate_factor)
                tmp_list[0] -= 64
                tmp_list[1] -= 64
                pixelate_factor = tuple(tmp_list)
            elif index >= (end_frame - pixelation_duration):
                # We are in decrementing process
                tmp_list = list(pixelate_factor)
                tmp_list[0] += 64
                tmp_list[1] += 64
                pixelate_factor = tuple(tmp_list)
        else:
            # Just copy the original frame
            shutil.copy(original_frames_array[index], tmp_frame_filename)
            
    ImageSequenceClip(processed_frames_array, fps = video_input.fps).set_audio(video_input.audio).write_videofile(output_filename)
        
    # TODO: Which one is faster, moviepy or ffmpeg (?)
    #parameters = ['ffmpeg', '-y', '-i', WIP_FOLDER + 'tmp_frame_processed_%01d.png', '-r', '30', '-pix_fmt', 'yuva420p', output_filename]
    #run(parameters)


def crop_audio_file(audio_filename: str, duration: float, output_filename: str):
    """
    Crops the 'audio_filename' provided to the requested 'duration'.

    This method returns the new audio 'output_filename' if valid, or
    False if it was not possible to crop it.
    """
    if not audio_filename:
        return None
    
    if not output_filename:
        return None

    from moviepy.editor import AudioFileClip

    audio_clip = AudioFileClip(audio_filename)

    if audio_clip.duration < duration:
        # TODO: Exception, None, review this
        print('audio is shorter than requested duration')
        return False
    
    audio_clip.set_duration(duration).write_audiofile(output_filename)

    return output_filename


from moviepy.editor import VideoFileClip, AudioFileClip
from yta_general_utils.file_processor import file_is_audio_file, file_is_video_file, rename_file
from yta_general_utils.tmp_processor import create_tmp_filename
from yta_general_utils.type_checker import variable_is_type
from typing import Union

import ffmpeg

def extract_audio_from_video(video_input: Union[VideoFileClip, str], output_filename: Union[str, None] = None):
    """
    Returns the audio in the video file provided as 'video_input'. If
    'output_filename' provided, it will write the audio in a file
    with that name.
    """
    if not video_input:
        return None
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
            
        video_input = VideoFileClip(video_input)

    if output_filename:
        # TODO: Check extension, please
        video_input.write_audiofile(output_filename)

    return video_input.audio

def set_audio_in_video(video_input: Union[VideoFileClip, str], audio_input: Union[AudioFileClip, str], output_filename: Union[str, None] = None):
    """
    This method returns a VideoFileClip that is the provided 'video_input' 
    with the also provided 'audio_input' as the unique audio (if valid
    parameters are provided). If 'output_filename' provided, it will
    write the video file with the new audio with that provided name.

    (!) If the input video file and the output file name are the same, you 
    will lose the original as it will be replaced.
    """
    if not video_input:
        return None
    
    if variable_is_type(video_input, str):
        if not file_is_video_file(video_input):
            return None
        
        video_input = VideoFileClip(video_input)
    
    if not audio_input:
        return None
    
    if variable_is_type(audio_input, str):
        if not file_is_audio_file(audio_input):
            return None
        
        audio_input = AudioFileClip(audio_input)

    video_input = video_input.set_audio(audio_input)

    if output_filename:
        # TODO: Check extension, please
        tmp_output_filename = create_tmp_filename('tmp_output.mp4')

        if variable_is_type(video_input, str) and variable_is_type(audio_input, str) and video_input and audio_input:
            # Both are valid str filenames, we will concat with ffmpeg
            # TODO: Check if I can get the original filename from an AudioFileClip|VideoFileClip
            ffmpeg.concat(ffmpeg.input(video_input), ffmpeg.input(audio_input), v = 1, a = 1).output(tmp_output_filename).run()
        else:
            video_input.write_videofile(tmp_output_filename)

        rename_file(tmp_output_filename, output_filename, True)

    return video_input


def remove_video_file_background(video_filename: str, output_filename: str):
    """
    Removes the background from the provided 'input_filename' and
    stores it in a new video as 'output_filename' file. The output
    file will be forced to have the '.mov' extension.
    """
    # TODO: This is too demanding as I cannot process it properly
    # Output must end in .mov to preserve transparency
    # TODO: Refactor this code to make it work with python code and not command
    if not video_filename:
        return None
    
    if not output_filename:
        return None
    
    if not output_filename.endswith('.mov'):
        output_filename += '.mov'

    from subprocess import run

    command_parameters = ['backgroundremover', '-i', video_filename, '-tv', '-o', output_filename]

    run(command_parameters)

    return output_filename